/*
 *  glassBall.cpp
 *  openFrameworks
 *
 *  Created by Mr. Magoo on 2/4/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */

#include "glassBall.h"

glassBall::glassBall()
{

}

void glassBall::setup(int x, int y, int w, int h, bool is_colorful)
{
	for(int i = 0; i < GLASS_BALL_MOVIE_LENGTH; i++)
	{
		face_img[i].setUseTexture(true);
		face_img[i].allocate(w, h);
	}
	
	// setup the bounding box
	coords.set(x, y, x+w, y+h);
	
	width = w;
	height = h;
	current_frame_to_load = 0;
	current_frame_to_play = -1;
	loaded = false;
	
	// once the mouse is over the bounding box, then the user can edit the glass ball boundaries
	editable = false;
	
	// show the glass ball
	show = true;
	colorful = is_colorful;
	
	num_frames = GLASS_BALL_MOVIE_LENGTH;
	random_counter = rand() % GLASS_BALL_MOVIE_LENGTH;
}

void glassBall::reset()
{
	loaded = false; 
	current_frame_to_play = -1; 
	current_frame_to_load = 0; 
	random_counter = rand() % GLASS_BALL_MOVIE_LENGTH;
}

void glassBall::setColorful(bool is_colorful)
{
	colorful = is_colorful;
}

void glassBall::addFrame(ofxCvColorImage &color_img, int x, int y, int w, int h)
{
	// 4 for not colorful, random 1-3 for r,g,b
	// 5,6,7 for sobel edges ? luminance ? contrast ? motion ?
	int channel_type;
	//face_img[current_frame_to_load].scaleRegionIntoMe(color_img, x, y, w, h, CV_INTER_NN);
	if(colorful)
	{
		channel_type = rand() % 4;
	}
	else
	{
		channel_type = 6;
	}
	//channel_type = rand() % 6;
	//channel_type = 3;
	//channel_type = (current_frame_to_load / (GLASS_BALL_MOVIE_LENGTH / 5));
	int bright = 256;
	//if(current_frame_to_load < GLASS_BALL_SEQUENCE_LENGTH)
	if(channel_type == 0)
	{
		face_img[current_frame_to_load].scaleRegionAndChannelIntoMe(color_img, x, y, w, h, 1, CV_INTER_NN);
	}
	//else if(current_frame_to_load < GLASS_BALL_SEQUENCE_LENGTH * 2)
	else if(channel_type == 1)
	{
		face_img[current_frame_to_load].scaleRegionAndChannelIntoMe(color_img, x, y, w, h, 2, CV_INTER_NN);
	}
	//else if(current_frame_to_load < GLASS_BALL_SEQUENCE_LENGTH * 3)
	else if(channel_type == 2)
	{
		face_img[current_frame_to_load].scaleRegionAndChannelIntoMe(color_img, x, y, w, h, 3, CV_INTER_NN);
	}
	else if(channel_type == 3 || channel_type == 4)
	{
		cvAddS(color_img.getCvImage(), cvScalar(bright-128,bright-128,bright-128), color_img.getCvImage());
		//cvScale(color_img.getCvImage(), color_img.getCvImage(), 5);
		cvSobel(color_img.getCvImage(), color_img.getCvImage(),1,1,5);
		face_img[current_frame_to_load].scaleRegionIntoMe(color_img, x, y, w, h, CV_INTER_NN);	
	}
	else
	{
		/*IplImage *edge_img = cvCreateImage( cvGetSize(color_img.getCvImage()), color_img.getCvImage()->depth, 3);
		 cvSobel(color_img.getCvImage(), edge_img, 1, 1, 5);
		 
		 face_img[current_frame_to_load].scaleRegionIntoMe(edge_img, x, y, w, h, CV_INTER_NN);
		 
		 cvReleaseImage( &edge_img );
		 */
		face_img[current_frame_to_load].scaleRegionIntoMe(color_img, x, y, w, h, CV_INTER_NN);		
	}
		
	if(current_frame_to_load > 0)
	{
		face_img[current_frame_to_load].average(face_img[current_frame_to_load-1]);
	}
	current_frame_to_load = (current_frame_to_load + 1) % GLASS_BALL_MOVIE_LENGTH;
	if(current_frame_to_load == 0)
		loaded = true;
}

ofxCvColorImage * glassBall::getNextFrame()
{
	random_counter = (random_counter + 1) % GLASS_BALL_MOVIE_LENGTH;
	current_frame_to_play = (current_frame_to_play + 1) % GLASS_BALL_MOVIE_LENGTH;
	return	&face_img[current_frame_to_play];
}