// parag k. mital
 /*
 
 add option to delete a glass ball or hide it
 show colors/edges feature binding idea...
 
 */
#include "testApp.h"
#include <math.h>

testApp::testApp()
{
	
}
testApp::~testApp()
{
}

//--------------------------------------------------------------
void testApp::setup(){	
	
	show_video = false;
	show_average = false;
	do_mean_shift = false;
	find_eyes = false;
	find_faces = true;
	do_colors = false;
	fire_neurons = true;
	
	ofSetWindowShape(SCREEN_WIDTH, SCREEN_HEIGHT);
	ofSetFrameRate(20);
	ofSetVerticalSync(true);
	ofSetFullscreen(true);
	
	vid_grabber[0].listDevices();
	vid_grabber[0].setVerbose(false);
	vid_grabber[0].setUseTexture(true);
    vid_grabber[0].initGrabber(OG_CAMERA_WIDTH,OG_CAMERA_HEIGHT);
	
#if SECOND_CAMERA
	vid_grabber[1].listDevices();
	vid_grabber[1].setVerbose(false);
	vid_grabber[1].setUseTexture(true);
    vid_grabber[1].initGrabber(OG_CAMERA_WIDTH,OG_CAMERA_HEIGHT);
#endif
	
	color_img_og.allocate(OG_CAMERA_WIDTH,OG_CAMERA_HEIGHT);
    color_img.allocate(CAMERA_WIDTH,CAMERA_HEIGHT);
	gray_image.allocate(CAMERA_WIDTH,CAMERA_HEIGHT);
	
	haar_finder.setup("haarXML/haarcascade_frontalface_alt2.xml");
	haar_finder_eyeL.setup("haarXML/ojoI.xml");
	haar_finder_eyeR.setup("haarXML/ojoD.xml");
	
	int x_offset = (SCREEN_WIDTH - ((int)(SCREEN_WIDTH / FACE_WIDTH) * (FACE_WIDTH))) / 2;
	int y_offset = (SCREEN_HEIGHT - ((int)(SCREEN_HEIGHT / FACE_HEIGHT) * (FACE_HEIGHT))) / 2;
	//int y_offset = (SCREEN_HEIGHT / 2);
	
	glass_balls_counter = 0;
	for(int i = 0; i < NUMBER_OF_GLASS_BALLS; i++)
	{
		int x = x_offset + (i % (int)(SCREEN_WIDTH / FACE_WIDTH))*FACE_WIDTH;
		int y = y_offset + (int)(FACE_WIDTH*(i+1) / SCREEN_WIDTH) * FACE_HEIGHT;
		printf("%d, %d\n", x, y);
		glass_balls[i].setup(x, y, FACE_WIDTH, FACE_HEIGHT, false);//rand() % 2);
	}
	// do all the glass balls have faces in them?
	loaded_all_glass_balls = 0;
	
	//shader.loadShader((char *)ofToDataPath("bleachbypass.fp.glsl").c_str(), (char *)ofToDataPath("bleachbypass.vp.glsl").c_str());
	shader.loadShader((char *)ofToDataPath("cc.bright.fp.glsl").c_str(), (char *)ofToDataPath("cc.brcosa.vp.glsl").c_str());
	shader.setUniform("brightness", 2.0f);

	shader.setShaderActive(false);
	
	fire_width = rand() % 10 + 1.0f;;
	
}

//--------------------------------------------------------------
void testApp::update(){
	ofBackground(0,0,0);
	    
    bool b_new_frame = false;
	
	for( int camera_num = 0; camera_num < NUM_CAMERAS; camera_num++ )
	{
		vid_grabber[camera_num].grabFrame();
		b_new_frame = vid_grabber[camera_num].isFrameNew();
		if (b_new_frame)
		{
			color_img_og.setFromPixels(vid_grabber[camera_num].getPixels(), OG_CAMERA_WIDTH, OG_CAMERA_HEIGHT);
			//color_img_og.mirror(true, false);
			color_img.scaleIntoMe(color_img_og, CV_INTER_NN);
			gray_image = color_img;
			//gray_image.mirror(true, false);
			//gray_image.rotate(20, 0, 0);
			//gray_image.equalizeHist();
			
			if(find_faces)
			{
				haar_finder.findHaarObjects(gray_image, 100, CAMERA_WIDTH * CAMERA_HEIGHT / 3, 6);
				for(int i = 0; i < haar_finder.blobs.size(); i++)
					//if(haar_finder.blobs.size() == 1)
				{
					float x = haar_finder.blobs[i].boundingRect.x;
					float y = haar_finder.blobs[i].boundingRect.y;
					float w = haar_finder.blobs[i].boundingRect.width;
					float h = haar_finder.blobs[i].boundingRect.height;
					
					//CvCapture *cap = new CvCaptureFromAVI("in.avi");
					
					// if we have loaded all the glass balls with faces,
					// then randomly change one
					if(loaded_all_glass_balls == 1)
					{
						int glass_balls_counter = rand() % NUMBER_OF_GLASS_BALLS;
						glass_balls[glass_balls_counter].reset();
						glass_balls[glass_balls_counter].setColorful(false);//rand() % 2);
						loaded_all_glass_balls = 2;
					}
					// else if we haven't loaded them all, check if we have finished loading one
					else if(glass_balls[glass_balls_counter].isLoaded())
					{
						// then start adding frames to another glass ball
						glass_balls_counter = (glass_balls_counter + 1) % NUMBER_OF_GLASS_BALLS;
						
						// check to see if all glass balls have faces in them
						if(glass_balls_counter == 0)
							loaded_all_glass_balls = 1;
					}
					// else stays the same glass ball
					
					// now add it to the current glass_ball_counter
					glass_balls[glass_balls_counter].addFrame(color_img_og, x*CAMERA_SCALE, y*CAMERA_SCALE, w*CAMERA_SCALE, h*CAMERA_SCALE);
					
				}	
			}
			
			if(find_eyes)
			{
				haar_finder_eyeL.findHaarObjects(gray_image, 100, CAMERA_WIDTH * CAMERA_HEIGHT / 6, 2);
				for(int i = 0; i < MIN(haar_finder_eyeL.blobs.size(),1); i++)
					//if(haar_finder.blobs.size() == 1)
				{
					float x = haar_finder_eyeL.blobs[i].boundingRect.x;
					float y = haar_finder_eyeL.blobs[i].boundingRect.y;
					float w = haar_finder_eyeL.blobs[i].boundingRect.width;
					float h = haar_finder_eyeL.blobs[i].boundingRect.height;
					
					//CvCapture *cap = new CvCaptureFromAVI("in.avi");
					
					// if we have loaded all the glass balls with faces,
					// then randomly change one
					if(loaded_all_glass_balls == 1)
					{
						int glass_balls_counter = rand() % NUMBER_OF_GLASS_BALLS;
						glass_balls[glass_balls_counter].reset();
						loaded_all_glass_balls = 2;
					}
					// else if we haven't loaded them all, check if we have finished loading one
					else if(glass_balls[glass_balls_counter].isLoaded())
					{
						// then start adding frames to another glass ball
						glass_balls_counter = (glass_balls_counter + 1) % NUMBER_OF_GLASS_BALLS;
						
						// check to see if all glass balls have faces in them
						if(glass_balls_counter == 0)
							loaded_all_glass_balls = 1;
					}
					// else stays the same glass ball
					
					// now add it to the current glass_ball_counter
					glass_balls[glass_balls_counter].addFrame(color_img_og, x*CAMERA_SCALE, y*CAMERA_SCALE, w*CAMERA_SCALE, h*CAMERA_SCALE);
					
				}
			}
			 
			
		}
	}
	if(fire_neurons)
	{
		if( (loaded_all_glass_balls == 2) && (ofGetSeconds() % 20 == 0))
		{
			for(int i = 0; i < NUMBER_OF_GLASS_BALLS; i++)
			{
				glass_balls[i].show = rand() % 2 == 0 ? true : false;
			}
			fire_width = rand() % 10 + 1.0f;;
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){

	ofBackground(0,0,0);
	ofSetColor(0xffffff);
	
	ofEnableAlphaBlending();
	
	ofNoFill();
	
	int x_offset = (SCREEN_WIDTH - ((int)(SCREEN_WIDTH / FACE_WIDTH) * (FACE_WIDTH))) / 2;
	int y_offset = (SCREEN_HEIGHT - ((int)(SCREEN_HEIGHT / FACE_HEIGHT) * (FACE_HEIGHT))) / 2;
	int y = SCREEN_HEIGHT / 2 - (FACE_HEIGHT / 2);
	
	int face_num = 0;
	int offset_face_num = MAX(0,glass_balls_counter - (SCREEN_WIDTH / FACE_WIDTH));
	
	int num_balls_to_display = (loaded_all_glass_balls > 0) ? (NUMBER_OF_GLASS_BALLS) : (glass_balls_counter);
	
	//for(int i = 0; i < MIN(num_balls_to_display, ((SCREEN_HEIGHT / FACE_HEIGHT) * (SCREEN_WIDTH / FACE_WIDTH))); i++)
#pragma omp parallel for
	for(int i = 0; i < num_balls_to_display; i++)
	{
		//glass_balls[i].getNextFrame()->draw(0, 0, glass_balls[i].width, glass_balls[i].height);
		if(glass_balls[i].show)
		{
			float brightness =  (float)glass_balls[i].random_counter / (float)glass_balls[i].num_frames * fire_width - (fire_width / 2.0f);
			brightness = exp(-(brightness*brightness)) * 1.8f;
			//float brightness = 1.75f;
			// draw with brightness
			shader.setShaderActive(true);
			//shader.setUniform("alpha", brightness);
			shader.setUniform("brightness", brightness);
			glass_balls[i].getNextFrame()->drawToCircle(glass_balls[i].coords.x1, glass_balls[i].coords.y1, glass_balls[i].width, glass_balls[i].height);
			shader.setShaderActive(false);
		}
		if(glass_balls[i].editable)
		{
			ofRect(glass_balls[i].coords.x1, glass_balls[i].coords.y1, glass_balls[i].width, glass_balls[i].height);
		}
	}
	
	// debug output
	if(show_video)
	{
		vid_grabber[0].draw(0,0,OG_CAMERA_WIDTH,OG_CAMERA_HEIGHT);
#if SECOND_CAMERA
		vid_grabber[1].draw(OG_CAMERA_WIDTH,0,OG_CAMERA_WIDTH,OG_CAMERA_HEIGHT);
#endif
		//color_img_og.draw(0,0);
		//gray_image.draw(0, 0);
		if(find_faces)
		{
			for(int i = 0; i < haar_finder.blobs.size(); i++){
				float x = haar_finder.blobs[i].boundingRect.x*CAMERA_SCALE;// + OG_CAMERA_WIDTH;
				float y = haar_finder.blobs[i].boundingRect.y*CAMERA_SCALE;
				float w = haar_finder.blobs[i].boundingRect.width*CAMERA_SCALE;
				float h = haar_finder.blobs[i].boundingRect.height*CAMERA_SCALE;
				
				ofSetColor(0xFF0000);
				ofRect(x, y, w, h);
			}
		}
		
		if(find_eyes)
		{
			for(int i = 0; i < haar_finder_eyeL.blobs.size(); i++){
				float x = haar_finder_eyeL.blobs[i].boundingRect.x*CAMERA_SCALE;
				float y = haar_finder_eyeL.blobs[i].boundingRect.y*CAMERA_SCALE;
				float w = haar_finder_eyeL.blobs[i].boundingRect.width*CAMERA_SCALE;
				float h = haar_finder_eyeL.blobs[i].boundingRect.height*CAMERA_SCALE;
				
				ofSetColor(0x00FF00);
				ofRect(x, y, w, h);
			}
		}
		
		/*for(int i = 0; i < haar_finder_profile.blobs.size(); i++){
			float x = haar_finder_profile.blobs[i].boundingRect.x;
			float y = haar_finder_profile.blobs[i].boundingRect.y;
			float w = haar_finder_profile.blobs[i].boundingRect.width;
			float h = haar_finder_profile.blobs[i].boundingRect.height;
			
			float cx = haar_finder_profile.blobs[i].centroid.x;
			float cy = haar_finder_profile.blobs[i].centroid.y;
			
			ofSetColor(0x00FF00);
			ofRect(x, y, w, h);
		}*/
		
		
		// draw the movie time as text above the movie time slider
		ofSetColor(0xFFFFFF);
		char reportString[255];
		sprintf(reportString, "faces: %d eyes: %d", find_faces, find_eyes);
		ofDrawBitmapString(reportString,40, 40);
		
	}
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	int num_balls_to_display = (loaded_all_glass_balls > 0) ? (NUMBER_OF_GLASS_BALLS) : (glass_balls_counter);
	switch (key){
		case ' ':
			break;
		case '+':
			for(int i = 0; i < num_balls_to_display; i++)
			{
				if(glass_balls[i].editable)
				{
					glass_balls[i].width++;
					break;
				}
			}
			break;
		case '_':
			for(int i = 0; i < num_balls_to_display; i++)
			{
				if(glass_balls[i].editable)
				{
					glass_balls[i].width--;
					break;
				}
			}
			break;
		case '=':
			for(int i = 0; i < num_balls_to_display; i++)
			{
				if(glass_balls[i].editable)
				{
					glass_balls[i].height++;
					break;
				}
			}
			break;
		case '-':
			for(int i = 0; i < num_balls_to_display; i++)
			{
				if(glass_balls[i].editable)
				{
					glass_balls[i].height--;
					break;
				}
			}
			break;
		case 'w':
		{
			// write positions to file:
			FILE *fid = fopen(ofToDataPath("positions.txt").c_str(), "w");
			for(int i = 0; i < num_balls_to_display; i++)
			{
				fprintf(fid, "%d %d %d %d\n", glass_balls[i].coords.x1, glass_balls[i].coords.y1, glass_balls[i].width, glass_balls[i].height);
			}
			fclose(fid);
			break;
		}
		case 'r':
		{
			// read positions from file:
			FILE *fid = fopen(ofToDataPath("positions.txt").c_str(), "r");
			for(int i = 0; i < num_balls_to_display; i++)
			{
				fscanf(fid, "%d %d %d %d", &glass_balls[i].coords.x1, &glass_balls[i].coords.y1, &glass_balls[i].width, &glass_balls[i].height);
			}
			fclose(fid);
			break;
		}
		case 'a':
			show_average = !show_average;
			break;
		case 's':
			vid_grabber[0].videoSettings();
#if SECOND_CAMERA
			vid_grabber[1].videoSettings();
#endif
			break;
		case 'f':
			ofToggleFullscreen();
			break;	
		case 'v':
			fire_neurons = !fire_neurons;
			if(!fire_neurons)
			{
				for(int i = 0; i < NUMBER_OF_GLASS_BALLS; i++)
				{
					glass_balls[i].show = true;
				}
			}
			break;
		case 'd':
			show_video = !show_video;
			break;
		case 'm':
			do_mean_shift = !do_mean_shift;
			break;
		case 'e':
			edit_projection = !edit_projection;
			break;
		case 'h':
			int num_balls_to_display = (loaded_all_glass_balls > 0) ? (NUMBER_OF_GLASS_BALLS) : (glass_balls_counter);
			for(int i = 0; i < num_balls_to_display; i++)
			{
				//if( x > glass_balls[i].coords.x1 && x < glass_balls[i].coords.x2 &&
				//   y > glass_balls[i].coords.y1 && y < glass_balls[i].coords.y2 )
				if(glass_balls[i].editable)
				{
					glass_balls[i].show = !glass_balls[i].show;
				}
			}
			break;
		case 'c': 
			do_colors = !do_colors;
			break;
		case '7':
			find_faces = !find_faces;
			break;
		case '8':
			find_eyes = !find_eyes;
			break;
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	int num_balls_to_display = (loaded_all_glass_balls > 0) ? (NUMBER_OF_GLASS_BALLS) : (glass_balls_counter);
	for(int i = 0; i < num_balls_to_display; i++)
	{
		if( x > glass_balls[i].coords.x1 && x < (glass_balls[i].coords.x1 + glass_balls[i].width) &&
		   y > glass_balls[i].coords.y1 && y < (glass_balls[i].coords.y1 + glass_balls[i].height) )
		{
			glass_balls[i].editable = true;
		}
		else
		{
			glass_balls[i].editable = false;
		}
	}
}	

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	int num_balls_to_display = (loaded_all_glass_balls > 0) ? (NUMBER_OF_GLASS_BALLS) : (glass_balls_counter);
	for(int i = 0; i < num_balls_to_display; i++)
	{
		//if( x > glass_balls[i].coords.x1 && x < glass_balls[i].coords.x2 &&
		//   y > glass_balls[i].coords.y1 && y < glass_balls[i].coords.y2 )
		if(glass_balls[i].editable)
		{
			glass_balls[i].coords.x1 = x - glass_balls[i].width / 2;
			glass_balls[i].coords.x2 = x + glass_balls[i].width / 2;
			glass_balls[i].coords.y1 = y - glass_balls[i].height / 2;
			glass_balls[i].coords.y2 = y + glass_balls[i].height / 2;
			break;
		}
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){

}

